@@ -1,6 +1,6 @@
 /* editor-document.c
  *
- * Copyright 2020 Christian Hergert <chergert@redhat.com>
+ * Copyright 2020-2023 Christian Hergert <chergert@redhat.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 #include <glib/gi18n.h>
 
-#include "editor-application.h"
+#include "editor-application-private.h"
 #include "editor-buffer-monitor-private.h"
 #include "editor-document-private.h"
 #include "editor-spell-checker.h"
@@ -435,7 +435,7 @@ editor_document_delete_range (GtkTextBuffer *buffer,
   g_assert (end != NULL);
   g_assert (gtk_text_iter_compare (start, end) <= 0);
 
-  /* Unlikely, but short circuit during loading incase something
+  /* Unlikely, but short circuit during loading in-case something
    * beneath us has to delete/replace as part of resolving invalid
    * characters.
    */
@@ -1125,6 +1125,36 @@ editor_document_save_cb (GObject      *object,
 
   if (!gtk_source_file_saver_save_finish (saver, result, &error))
     {
+      g_autofree char *uri = g_file_get_uri (file);
+
+      /* Try again as admin:// */
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED) &&
+          g_str_has_prefix (uri, "file://"))
+        {
+          g_autofree char *admin_uri = g_strdup_printf ("admin://%s", uri + strlen ("file://"));
+          g_autoptr(GFile) admin_file = g_file_new_for_uri (admin_uri);
+          g_autoptr(GtkSourceFileSaver) admin_saver = NULL;
+
+          gtk_source_file_set_location (source_file, admin_file);
+
+          admin_saver = gtk_source_file_saver_new (GTK_SOURCE_BUFFER (self), source_file);
+          gtk_source_file_saver_set_compression_type (admin_saver, gtk_source_file_saver_get_compression_type (saver));
+          gtk_source_file_saver_set_encoding (admin_saver, gtk_source_file_saver_get_encoding (saver));
+          gtk_source_file_saver_set_flags (admin_saver, gtk_source_file_saver_get_flags (saver));
+          gtk_source_file_saver_set_newline_type (admin_saver, gtk_source_file_saver_get_newline_type (saver));
+
+          gtk_source_file_saver_save_async (admin_saver,
+                                            G_PRIORITY_DEFAULT,
+                                            NULL,
+                                            editor_document_progress,
+                                            self,
+                                            NULL,
+                                            editor_document_save_cb,
+                                            g_steal_pointer (&task));
+
+          return;
+        }
+
       _editor_document_unmark_busy (self);
       g_task_return_error (task, g_steal_pointer (&error));
       return;
@@ -1375,9 +1405,42 @@ editor_document_query_info_cb (GObject      *object,
   gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (self), language);
   gtk_source_buffer_set_highlight_syntax (GTK_SOURCE_BUFFER (self), language != NULL);
 
-  /* Parse metadata for cursor position */
-  if (position != NULL &&
-      sscanf (position, "%u:%u", &line, &line_offset) >= 1)
+  /* If the cursor position was provided at app startup, then steal
+   * that position from the hashtable and restore it.
+   *
+   * Otherwise, if the metadata for the file has the position, use that
+   * so that the session looks similar to when we closed.
+   */
+  if (_editor_application_consume_position (EDITOR_APPLICATION_DEFAULT, file, &line, &line_offset))
+    {
+      GtkTextIter iter;
+
+      g_debug ("Moving cursor to requested position %u:%u",
+               line, line_offset);
+
+      if (line == 0)
+        gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (self), &iter);
+      else
+        gtk_text_buffer_get_iter_at_line_offset (GTK_TEXT_BUFFER (self),
+                                                 &iter,
+                                                 line - 1,
+                                                 line_offset > 0 ? line_offset - 1 : 0);
+
+      /* Move to first non-space character if offset is not provided */
+      if (line_offset == 0)
+        {
+          while (!gtk_text_iter_ends_line (&iter) &&
+                 g_unichar_isspace (gtk_text_iter_get_char (&iter)))
+            {
+              if (!gtk_text_iter_forward_char (&iter))
+                break;
+            }
+        }
+
+      gtk_text_buffer_select_range (GTK_TEXT_BUFFER (self), &iter, &iter);
+    }
+  else if (position != NULL &&
+           sscanf (position, "%u:%u", &line, &line_offset) >= 1)
     {
       GtkTextIter iter;
 
@@ -1552,7 +1615,7 @@ editor_document_do_load (EditorDocument *self,
     }
 
   gtk_source_file_loader_load_async (loader,
-                                     G_PRIORITY_DEFAULT,
+                                     G_PRIORITY_LOW,
                                      g_task_get_cancellable (task),
                                      editor_document_progress,
                                      self,
