@@ -24,7 +24,7 @@
 
 #include <glib/gstdio.h>
 
-#include "editor-application.h"
+#include "editor-application-private.h"
 #include "editor-document-private.h"
 #include "editor-page-private.h"
 #include "editor-session-private.h"
@@ -44,6 +44,7 @@ typedef struct
   GPtrArray    *seen;
   GPtrArray    *forgot;
   guint         n_active;
+  guint         remember_recent_files : 1;
 } EditorSessionSave;
 
 typedef struct
@@ -88,6 +89,15 @@ static guint signals[N_SIGNALS];
 static guint default_width;
 static guint default_height;
 
+static GSettings *
+find_g_settings (const char* schema_id)
+{
+  GSettingsSchemaSource *source = g_settings_schema_source_get_default ();
+  g_autoptr(GSettingsSchema) schema = g_settings_schema_source_lookup (source, schema_id, TRUE);
+
+  return schema != NULL ? g_settings_new (schema_id) : NULL;
+}
+
 static void
 selection_free (Selection *selection)
 {
@@ -371,6 +381,7 @@ editor_session_dispose (GObject *object)
     g_ptr_array_remove_range (self->windows, 0, self->windows->len);
 
   g_clear_handle_id (&self->auto_save_source, g_source_remove);
+  g_clear_object (&self->privacy);
 
   G_OBJECT_CLASS (editor_session_parent_class)->dispose (object);
 }
@@ -608,6 +619,8 @@ editor_session_init (EditorSession *self)
                                                 NULL);
   self->drafts = g_array_new (FALSE, FALSE, sizeof (EditorSessionDraft));
   g_array_set_clear_func (self->drafts, (GDestroyNotify) clear_draft);
+
+  self->privacy = find_g_settings ("org.gnome.desktop.privacy");
 }
 
 EditorSession *
@@ -1090,7 +1103,6 @@ editor_session_update_recent_worker (GTask        *task,
                                      GCancellable *cancellable)
 {
   EditorSessionSave *save = task_data;
-  g_autoptr(GSettings) settings = NULL;
   g_autoptr(GBookmarkFile) bookmarks = NULL;
   g_autofree gchar *filename = NULL;
   g_autoptr(GError) error = NULL;
@@ -1100,9 +1112,7 @@ editor_session_update_recent_worker (GTask        *task,
   g_assert (save != NULL);
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  settings = g_settings_new ("org.gnome.desktop.privacy");
-
-  if (!g_settings_get_boolean (settings, "remember-recent-files"))
+  if (!save->remember_recent_files)
     {
       /* Just delete recent files if the user doesn't want them */
       g_autofree gchar *path = get_bookmarks_filename ();
@@ -1266,6 +1276,8 @@ editor_session_save_async (EditorSession       *self,
   state->state_file = g_file_dup (self->state_file);
   state->state_bytes = g_variant_get_data_as_bytes (vstate);
   state->app = g_application_get_default ();
+  if (self->privacy && g_settings_get_boolean (self->privacy, "remember-recent-files"))
+    state->remember_recent_files = TRUE;
 
   if (g_hash_table_size (self->seen) > 0)
     {
@@ -1647,6 +1659,8 @@ editor_session_restore_v1_pages (EditorSession *self,
       const char *uri;
       gboolean is_active;
       Selection sel;
+      guint line;
+      guint line_offset;
 
       if (!g_variant_lookup (page, "draft-id", "&s", &draft_id))
         draft_id = NULL;
@@ -1676,6 +1690,19 @@ editor_session_restore_v1_pages (EditorSession *self,
       if (uri != NULL)
         file = g_file_new_for_uri (uri);
 
+      /* If the file was also requested from command line arguments
+       * and a +line:column was requested, prefer that to session state.
+       */
+      if (file != NULL &&
+          _editor_application_consume_position (EDITOR_APPLICATION_DEFAULT,
+                                                file,
+                                                &line,
+                                                &line_offset))
+        {
+          sel.end.line = sel.begin.line = MAX (1, line) - 1;
+          sel.end.line_offset = sel.begin.line_offset = MAX (1, line_offset) - 1;
+        }
+
       document = _editor_document_new (file, draft_id);
 
       if (encoding != NULL)
